@@ -19,7 +19,7 @@ from xpresso.bodies import (
     Json,
     Multipart,
 )
-from xpresso.datastructures import BinaryStream, UploadFile
+from xpresso.datastructures import UploadFile
 from xpresso.dependencies import Depends
 from xpresso.exception_handlers import ExcHandler
 from xpresso.parameters import (
@@ -40,7 +40,6 @@ from xpresso.routing.websockets import WebSocketRoute
 from xpresso.websockets import WebSocket
 
 __all__ = (
-    "BinaryStream",
     "ExcHandler",
     "Operation",
     "Path",
