@@ -1,9 +1,11 @@
 import contextlib
+import functools
 import inspect
 import typing
 
 import starlette.types
 from di.api.dependencies import DependantBase
+from di.api.solved import SolvedDependant
 from di.container import Container, ContainerState, bind_by_type
 from di.dependant import Dependant, JoinedDependant
 from di.executors import AsyncExecutor
@@ -94,43 +96,64 @@ class App:
 
         @contextlib.asynccontextmanager
         async def lifespan_ctx(*_: typing.Any) -> typing.AsyncIterator[None]:
-            lifespans, lifespan_deps = self._setup()
+            # first run setup to find all routes, their lifespans and callbacks to solve them
+            lifespans, prepare_cbs = self._setup()
             self._setup_run = True
-            original_container = self.container
+            placeholder = Dependant(lambda: None, scope="app")
+            dep: "DependantBase[typing.Any]"
+            executor = AsyncExecutor()
             async with self._container_state.enter_scope(
                 "app"
             ) as self._container_state:
+                # now solve and execute all lifespans
+                # lifespans can get a reference to the container and create/replace binds
+                # so it is important that we execute them before solving the endpoints
                 if lifespan is not None:
                     dep = Dependant(
                         _wrap_lifespan_as_async_generator(lifespan), scope="app"
                     )
                 else:
-
-                    async def null_lifespan() -> typing.AsyncIterator[None]:
-                        yield
-
-                    dep = Dependant(null_lifespan, scope="app")
+                    dep = placeholder
                 solved = self.container.solve(
                     JoinedDependant(
                         dep,
                         siblings=[
-                            *(
-                                Dependant(lifespan, scope="app")
-                                for lifespan in lifespans
-                            ),
-                            *lifespan_deps,
+                            Dependant(lifespan, scope="app") for lifespan in lifespans
                         ],
                     ),
                     scopes=Scopes,
                 )
                 try:
                     await self.container.execute_async(
-                        solved, executor=AsyncExecutor(), state=self._container_state
+                        solved, executor=executor, state=self._container_state
+                    )
+                    # now we can solve the endpoints
+                    # we accumulate any endpoint dependencies that are part of the "app"
+                    # scope and execute them immediately so that their setup and teardown
+                    # run in the same task
+                    # (the server will create separate tasks for the lifespan and endpoint,
+                    # if we run app scoped dependencies lazily the setup would run in a different
+                    # scope than the teardown)
+                    lifespan_deps: "typing.List[DependantBase[typing.Any]]" = []
+                    for cb in prepare_cbs:
+                        prepared = cb()
+                        lifespan_deps.extend(
+                            d for d in prepared.dag if d.scope == "app"
+                        )
+                    await self.container.execute_async(
+                        self.container.solve(
+                            JoinedDependant(
+                                placeholder,
+                                siblings=lifespan_deps,
+                            ),
+                            scopes=Scopes,
+                        ),
+                        executor,
+                        state=self._container_state,
                     )
                     yield
                 finally:
-                    # make this cm reentrant for testing purposes
-                    self.container = original_container
+                    # make this context manager reentrant for testing purposes
                     self._setup_run = False
                     self._container_state = ContainerState()
 
@@ -189,7 +212,9 @@ class App:
             return
         # http or websocket
         if not self._setup_run:
-            self._setup()
+            *_, prepare_callbacks = self._setup()
+            for cb in prepare_callbacks:
+                cb()
         if "extensions" not in scope:
             scope["extensions"] = extensions = {}
         else:
@@ -210,11 +235,13 @@ class App:
         self,
     ) -> typing.Tuple[
         typing.List[typing.Callable[..., typing.AsyncIterator[None]]],
-        typing.List[DependantBase[typing.Any]],
+        typing.List[typing.Callable[[], SolvedDependant[typing.Any]]],
     ]:
-        lifespans: typing.List[typing.Callable[..., typing.AsyncIterator[None]]] = []
-        lifespan_dependants: typing.List[DependantBase[typing.Any]] = []
-        seen_routers: typing.Set[typing.Any] = set()
+        lifespans: "typing.List[typing.Callable[..., typing.AsyncIterator[None]]]" = []
+        seen_routers: "typing.Set[typing.Any]" = set()
+        prepare_cbs: "typing.List[typing.Callable[[], SolvedDependant[typing.Any]]]" = (
+            []
+        )
         for route in visit_routes(
             app_type=App, router=self.router, nodes=[self, self.router], path=""
         ):
@@ -232,29 +259,29 @@ class App:
                         )
             if isinstance(route.route, Path):
                 for operation in route.route.operations.values():
-                    operation.prepare(
+                    prepare_cbs.append(
+                        functools.partial(
+                            operation.prepare,
+                            dependencies=[
+                                *dependencies,
+                                *route.route.dependencies,
+                                *operation.dependencies,
+                            ],
+                            container=self.container,
+                        )
+                    )
+            elif isinstance(route.route, WebSocketRoute):
+                prepare_cbs.append(
+                    functools.partial(
+                        route.route.prepare,
                         dependencies=[
                             *dependencies,
                             *route.route.dependencies,
-                            *operation.dependencies,
                         ],
                         container=self.container,
                     )
-                    for dep in operation.dependant.get_flat_subdependants():
-                        if dep.scope == "app":
-                            lifespan_dependants.append(dep)
-            elif isinstance(route.route, WebSocketRoute):
-                route.route.prepare(
-                    dependencies=[
-                        *dependencies,
-                        *route.route.dependencies,
-                    ],
-                    container=self.container,
                 )
-                for dep in route.route.dependant.get_flat_subdependants():
-                    if dep.scope == "app":
-                        lifespan_dependants.append(dep)
-        return lifespans, lifespan_dependants
+        return lifespans, prepare_cbs
 
     def get_openapi(
         self, servers: typing.List[openapi_models.Server]
