@@ -4,12 +4,7 @@ import typing
 import xpresso.openapi.models as openapi_models
 from xpresso._utils.pydantic_utils import model_field_from_param
 from xpresso._utils.typing import Annotated, get_args, get_origin
-from xpresso.binders.api import (
-    ModelNameMap,
-    OpenAPIMetadata,
-    SupportsExtractor,
-    SupportsOpenAPI,
-)
+from xpresso.binders.api import ModelNameMap, SupportsExtractor, SupportsOpenAPI
 from xpresso.binders.dependants import Binder, BinderMarker
 from xpresso.exceptions import HTTPException, RequestValidationError
 from xpresso.requests import HTTPConnection, Request
@@ -43,22 +38,14 @@ class BodyOpenAPI(typing.NamedTuple):
     def get_models(self) -> typing.List[type]:
         return [model for p in self.providers for model in p.get_models()]
 
-    def get_openapi(self, model_name_map: ModelNameMap) -> OpenAPIMetadata:
-        content: typing.Dict[str, openapi_models.MediaType] = {}
-        schemas: typing.Dict[str, typing.Any] = {}
+    def modify_operation_schema(
+        self,
+        model_name_map: ModelNameMap,
+        operation: openapi_models.Operation,
+        components: openapi_models.Components,
+    ) -> None:
         for provider in self.providers:
-            meta = provider.get_openapi(model_name_map)
-            if meta.body is None:
-                raise TypeError()
-            for k, v in meta.body.content.items():
-                if k in content:
-                    raise ValueError(f"Duplicate bodies for media type {k}")
-                content[k] = v
-            schemas.update(meta.schemas or {})
-        body = openapi_models.RequestBody(
-            required=self.required, content=content, description=self.description
-        )
-        return OpenAPIMetadata(body=body, schemas=schemas)
+            provider.modify_operation_schema(model_name_map, operation, components)
 
 
 class BodyOpenAPIMarker(typing.NamedTuple):
