@@ -3,11 +3,12 @@ import typing
 import starlette.routing
 import starlette.types
 from di.api.dependencies import DependantBase
+from di.api.providers import DependencyProvider
 from di.api.providers import DependencyProvider as Endpoint
 
 import xpresso.binders.dependants as dependants
 import xpresso.openapi.models as openapi_models
-from xpresso.dependencies import Depends
+from xpresso.dependencies._dependencies import DependsMarker
 from xpresso.responses import ResponseSpec, ResponseStatusCode
 from xpresso.routing.operation import Operation
 
@@ -42,7 +43,11 @@ class Path(starlette.routing.Route):
         trace: typing.Optional[typing.Union[Operation, Endpoint]] = None,
         redirect_slashes: bool = True,
         dependencies: typing.Optional[
-            typing.Iterable[typing.Union[DependantBase[typing.Any], Depends]]
+            typing.Iterable[
+                typing.Union[
+                    DependantBase[typing.Any], DependsMarker[DependencyProvider]
+                ]
+            ]
         ] = None,
         # OpenAPI metadata
         include_in_schema: bool = True,
@@ -61,7 +66,7 @@ class Path(starlette.routing.Route):
         self.path = path
         self.redirect_slashes = redirect_slashes
         self.dependencies = tuple(
-            dep if not isinstance(dep, Depends) else dep.as_dependant()
+            dep if isinstance(dep, DependantBase) else dep.as_dependant()
             for dep in dependencies or ()
         )
         self.summary = summary
@@ -72,7 +77,7 @@ class Path(starlette.routing.Route):
         self.responses = dict(responses or {})
 
         operations: typing.Dict[str, Operation] = {}
-        for param, operation in (
+        for operation_or_endpoint, method in (
             (get, "GET"),
             (head, "HEAD"),
             (post, "POST"),
@@ -83,9 +88,11 @@ class Path(starlette.routing.Route):
             (options, "OPTIONS"),
             (trace, "TRACE"),
         ):
-            if param:
-                operations[operation] = (
-                    param if isinstance(param, Operation) else Operation(param)
+            if operation_or_endpoint:
+                operations[method] = (
+                    operation_or_endpoint
+                    if isinstance(operation_or_endpoint, Operation)
+                    else Operation(operation_or_endpoint)
                 )
         self.operations = operations
         super().__init__(  # type: ignore  # for Pylance
